"""Constants used in libhomeseer."""

DEFAULT_ASCII_PORT = 11000
DEFAULT_HTTP_PORT = 80
DEFAULT_PASSWORD = "default"
DEFAULT_USERNAME = "default"

DEVICE_ZWAVE_BARRIER_OPERATOR = "Z-Wave Barrier Operator"
DEVICE_ZWAVE_BATTERY = "Z-Wave Battery"
DEVICE_ZWAVE_CENTRAL_SCENE = "Z-Wave Central Scene"
DEVICE_ZWAVE_DOOR_LOCK = "Z-Wave Door Lock"
DEVICE_ZWAVE_DOOR_LOCK_LOGGING = "Z-Wave Door Lock Logging"
DEVICE_ZWAVE_ELECTRIC_METER = "Z-Wave Electric Meter"
DEVICE_ZWAVE_FAN_STATE = "Z-Wave Fan State"
DEVICE_ZWAVE_LUMINANCE = "Z-Wave Luminance"
DEVICE_ZWAVE_OPERATING_STATE = "Z-Wave Operating State"
DEVICE_ZWAVE_RELATIVE_HUMIDITY = "Z-Wave Relative Humidity"
DEVICE_ZWAVE_SENSOR_BINARY = "Z-Wave Sensor Binary"
DEVICE_ZWAVE_SENSOR_MULTILEVEL = "Z-Wave Sensor Multilevel"
DEVICE_ZWAVE_SWITCH = "Z-Wave Switch"
DEVICE_ZWAVE_SWITCH_BINARY = "Z-Wave Switch Binary"
DEVICE_ZWAVE_SWITCH_MULTILEVEL = "Z-Wave Switch Multilevel"
DEVICE_ZWAVE_TEMPERATURE = "Z-Wave Temperature"
DEVICE_ZWAVE_WATTS = "Z-Wave Watts"

RELATIONSHIP_ROOT = 2
RELATIONSHIP_STANDALONE = 3
RELATIONSHIP_CHILD = 4
